@@ -23,6 +23,7 @@
 #include <KMimeType>
 #include <KColorScheme>
 #include <KColorUtils>
+#include <klocale.h>
 
 #include <ktexteditor/document.h>
 
@@ -452,8 +453,14 @@ QVariant KateFileTreeModel::data( const QModelIndex &index, int role ) const
     case Qt::DecorationRole:
       return item->icon();
 
-    case Qt::ToolTipRole:
-      return item->path();
+    case Qt::ToolTipRole: {
+      QString tooltip = item->path();
+      if (item->flag(ProxyItem::DeletedExternally) || item->flag(ProxyItem::ModifiedExternally)) {
+        tooltip = i18nc("%1 is the full path", "<p><b>%1</b></p><p>The document has been modified by another application.</p>").arg(item->path());
+      }
+
+      return tooltip;
+    }
 
     case Qt::ForegroundRole: {
       KColorScheme colors(QPalette::Active);
@@ -621,8 +628,8 @@ void KateFileTreeModel::documentOpened(KTextEditor::Document *doc)
   connect(doc, SIGNAL(documentNameChanged(KTextEditor::Document*)), this, SLOT(documentNameChanged(KTextEditor::Document*)));
   connect(doc, SIGNAL(documentUrlChanged(KTextEditor::Document*)), this, SLOT(documentNameChanged(KTextEditor::Document*)));
   connect(doc, SIGNAL(modifiedChanged(KTextEditor::Document*)), this, SLOT(documentModifiedChanged(KTextEditor::Document*)));
-  connect(doc, SIGNAL(modifiedOnDisk( KTextEditor::Document*, bool, KTextEditor::ModificationInterface::ModifiedOnDiskReason ) ),
-          this,  SLOT(documentModifiedOnDisc( KTextEditor::Document*, bool, KTextEditor::ModificationInterface::ModifiedOnDiskReason ) ) );
+  connect(doc, SIGNAL(modifiedOnDisk(KTextEditor::Document*,bool,KTextEditor::ModificationInterface::ModifiedOnDiskReason)),
+          this,  SLOT(documentModifiedOnDisc(KTextEditor::Document*,bool,KTextEditor::ModificationInterface::ModifiedOnDiskReason)) );
 
   kDebug(debugArea()) << "after add:" << item;
   
@@ -742,7 +749,6 @@ class EditViewCount
     int view;
 };
 
-// shamelessly "borrowed" from KateViewDocumentProxyModel
 void KateFileTreeModel::updateBackgrounds(bool force)
 {
   if (!m_shadingEnabled && !force) return;
