@@ -81,6 +81,7 @@ KatePluginGDBView::KatePluginGDBView( Kate::MainWindow* mainWin, Kate::Applicati
     m_lastExecFrame = 0;
     m_kateApplication = application;
     m_focusOnInput = true;
+    m_activeThread = -1;
 
 
     m_toolView = mainWindow()->createToolView(i18n("Debug View"),
@@ -116,7 +117,7 @@ KatePluginGDBView::KatePluginGDBView( Kate::MainWindow* mainWin, Kate::Applicati
 
     // input
     m_inputArea = new KHistoryComboBox( true );
-    connect( m_inputArea,  SIGNAL( returnPressed() ), this, SLOT( slotSendCommand() ) );
+    connect( m_inputArea,  SIGNAL(returnPressed()), this, SLOT(slotSendCommand()) );
     QHBoxLayout *inputLayout = new QHBoxLayout();
     inputLayout->addWidget( m_inputArea, 10 );
     inputLayout->setContentsMargins( 0,0,0,0 );
@@ -131,7 +132,15 @@ KatePluginGDBView::KatePluginGDBView( Kate::MainWindow* mainWin, Kate::Applicati
     layout->setSpacing(0);
 
     // stack page
-    m_stackTree = new QTreeWidget(m_stackToolView);
+    QWidget *stackContainer = new QWidget(m_stackToolView);
+    QVBoxLayout *stackLayout = new QVBoxLayout( stackContainer );
+    m_threadCombo = new QComboBox();
+    m_stackTree = new QTreeWidget();
+    stackLayout->addWidget( m_threadCombo );
+    stackLayout->addWidget( m_stackTree );
+    stackLayout->setStretch(0, 10);
+    stackLayout->setContentsMargins(0,0,0,0);
+    stackLayout->setSpacing(0);
     QStringList headers;
     headers << "  " << i18nc( "Column label (frame number)", "Nr" ) << i18nc( "Column label", "Frame" );
     m_stackTree->setHeaderLabels(headers);
@@ -139,62 +148,67 @@ KatePluginGDBView::KatePluginGDBView( Kate::MainWindow* mainWin, Kate::Applicati
     m_stackTree->resizeColumnToContents(0);
     m_stackTree->resizeColumnToContents(1);
     m_stackTree->setAutoScroll(false);
-    connect( m_stackTree, SIGNAL( itemActivated ( QTreeWidgetItem *, int ) ),
-             this, SLOT( stackFrameSelected() ) );
+    connect( m_stackTree, SIGNAL(itemActivated(QTreeWidgetItem*,int)),
+             this, SLOT(stackFrameSelected()) );
+
+    connect( m_threadCombo, SIGNAL(currentIndexChanged(int)),
+             this, SLOT(threadSelected(int)) );
+
 
     // config page
     m_configView = new ConfigView( NULL, mainWin );
 
     m_ioView = new IOView();
-    connect( m_configView, SIGNAL( showIO( bool ) ),
-             this,       SLOT( showIO( bool ) ) );
+    connect( m_configView, SIGNAL(showIO(bool)),
+             this,       SLOT(showIO(bool)) );
 
     m_tabWidget->addTab( m_gdbPage, i18nc( "Tab label", "GDB Output" ) );
-    //m_tabWidget->addTab( m_stackTree, i18nc( "Tab label", "Call Stack" ) );
     m_tabWidget->addTab( m_configView, i18nc( "Tab label", "Settings" ) );
-    //tabWidget->addTab( m_ioView, i18n( "IO" ) );
 
     m_localsView = new LocalsView(m_localsToolView);
 
     m_debugView  = new DebugView( this );
-    connect( m_debugView, SIGNAL( readyForInput( bool ) ),
-             this,        SLOT( enableDebugActions( bool ) ) );
+    connect( m_debugView, SIGNAL(readyForInput(bool)),
+             this,        SLOT(enableDebugActions(bool)) );
 
-    connect( m_debugView,  SIGNAL( outputText( const QString ) ),
-             m_outputArea, SLOT( append( const QString ) ) );
+    connect( m_debugView, SIGNAL(outputText(QString)),
+             this,        SLOT(addOutputText(QString)) );
 
-    connect( m_debugView, SIGNAL( outputError( const QString ) ),
-             this,        SLOT( addErrorText( const QString ) ) );
+    connect( m_debugView, SIGNAL(outputError(QString)),
+             this,        SLOT(addErrorText(QString)) );
 
-    connect( m_debugView, SIGNAL( debugLocationChanged( const KUrl, int ) ),
-             this,        SLOT( slotGoTo( const KUrl, int ) ) );
+    connect( m_debugView, SIGNAL(debugLocationChanged(KUrl,int)),
+             this,        SLOT(slotGoTo(KUrl,int)) );
 
-    connect( m_debugView, SIGNAL( breakPointSet( const KUrl, int ) ),
-             this,        SLOT( slotBreakpointSet( const KUrl, int ) ) );
+    connect( m_debugView, SIGNAL(breakPointSet(KUrl,int)),
+             this,        SLOT(slotBreakpointSet(KUrl,int)) );
 
-    connect( m_debugView, SIGNAL( breakPointCleared( const KUrl, int ) ),
-             this,        SLOT( slotBreakpointCleared( const KUrl, int ) ) );
+    connect( m_debugView, SIGNAL(breakPointCleared(KUrl,int)),
+             this,        SLOT(slotBreakpointCleared(KUrl,int)) );
 
-    connect( m_debugView, SIGNAL( clearBreakpointMarks() ),
-             this,        SLOT( clearMarks() ) );
+    connect( m_debugView, SIGNAL(clearBreakpointMarks()),
+             this,        SLOT(clearMarks()) );
 
-    connect( m_debugView, SIGNAL( programEnded() ),
-             this,        SLOT( programEnded() ) );
+    connect( m_debugView, SIGNAL(programEnded()),
+             this,        SLOT(programEnded()) );
 
-    connect( m_debugView, SIGNAL( gdbEnded() ),
-             this,        SLOT( programEnded() ) );
+    connect( m_debugView, SIGNAL(gdbEnded()),
+             this,        SLOT(programEnded()) );
 
-    connect( m_debugView, SIGNAL( gdbEnded() ),
-             this,        SLOT( gdbEnded() ) );
+    connect( m_debugView, SIGNAL(gdbEnded()),
+             this,        SLOT(gdbEnded()) );
 
-    connect( m_debugView, SIGNAL( stackFrameInfo( QString, QString ) ),
-             this,        SLOT( insertStackFrame( QString, QString ) ) );
+    connect( m_debugView, SIGNAL(stackFrameInfo(QString,QString)),
+             this,        SLOT(insertStackFrame(QString,QString)) );
 
-    connect( m_debugView, SIGNAL( stackFrameChanged( int ) ),
-             this,        SLOT( stackFrameChanged( int ) ) );
+    connect( m_debugView, SIGNAL(stackFrameChanged(int)),
+             this,        SLOT(stackFrameChanged(int)) );
 
-    connect( m_debugView,  SIGNAL( infoLocal( QString ) ),
-             m_localsView, SLOT( addLocal( QString ) ) );
+    connect( m_debugView,  SIGNAL(infoLocal(QString)),
+             m_localsView, SLOT(addLocal(QString)) );
+
+    connect( m_debugView, SIGNAL(threadInfo(int,bool)),
+             this,        SLOT(insertThread(int,bool)) );
 
     // Actions
     m_configView->registerActions( actionCollection() );
@@ -202,81 +216,81 @@ KatePluginGDBView::KatePluginGDBView( Kate::MainWindow* mainWin, Kate::Applicati
     KAction* a = actionCollection()->addAction( "debug" );
     a->setText( i18n( "Start Debugging" ) );
     a->setIcon( KIcon( "debug" ) );
-    connect(    a,      SIGNAL( triggered( bool ) ),
-                this,   SLOT( slotDebug() ) );
+    connect(    a,      SIGNAL(triggered(bool)),
+                this,   SLOT(slotDebug()) );
 
     a = actionCollection()->addAction( "kill" );
     a->setText( i18n( "Kill / Stop Debugging" ) );
     a->setIcon( KIcon( "media-playback-stop" ) );
-    connect(    a,         SIGNAL( triggered( bool ) ),
-                m_debugView, SLOT( slotKill() ) );
+    connect(    a,         SIGNAL(triggered(bool)),
+                m_debugView, SLOT(slotKill()) );
 
     a = actionCollection()->addAction( "rerun" );
     a->setText( i18n( "Restart Debugging" ) );
     a->setIcon( KIcon( "view-refresh" ) );
-    connect(    a,         SIGNAL( triggered( bool ) ),
-                this, SLOT( slotRestart() ) );
+    connect(    a,         SIGNAL(triggered(bool)),
+                this, SLOT(slotRestart()) );
 
     a = actionCollection()->addAction( "toggle_breakpoint" );
     a->setText( i18n( "Toggle Breakpoint / Break" ) );
     a->setIcon( KIcon( "media-playback-pause" ) );
-    connect(    a,      SIGNAL( triggered( bool ) ),
-                this,   SLOT( slotToggleBreakpoint() ) );
+    connect(    a,      SIGNAL(triggered(bool)),
+                this,   SLOT(slotToggleBreakpoint()) );
 
     a = actionCollection()->addAction( "step_in" );
     a->setText( i18n( "Step In" ) );
     a->setIcon( KIcon( "debug-step-into" ) );
-    connect(    a,      SIGNAL( triggered( bool ) ),
-                m_debugView,   SLOT( slotStepInto() ) );
+    connect(    a,      SIGNAL(triggered(bool)),
+                m_debugView,   SLOT(slotStepInto()) );
 
     a = actionCollection()->addAction( "step_over" );
     a->setText( i18n( "Step Over" ) );
     a->setIcon( KIcon( "debug-step-over" ) );
-    connect(    a,      SIGNAL( triggered( bool ) ),
-                m_debugView,   SLOT( slotStepOver() ) );
+    connect(    a,      SIGNAL(triggered(bool)),
+                m_debugView,   SLOT(slotStepOver()) );
 
     a = actionCollection()->addAction( "step_out" );
     a->setText( i18n( "Step Out" ) );
     a->setIcon( KIcon( "debug-step-out" ) );
-    connect(    a,      SIGNAL( triggered( bool ) ),
-                m_debugView, SLOT( slotStepOut() ) );
+    connect(    a,      SIGNAL(triggered(bool)),
+                m_debugView, SLOT(slotStepOut()) );
 
     a = actionCollection()->addAction( "move_pc" );
     a->setText( i18nc( "Move Program Counter (next execution)", "Move PC" ) );
-    connect(    a,      SIGNAL( triggered( bool ) ),
-                this,   SLOT( slotMovePC() ) );
+    connect(    a,      SIGNAL(triggered(bool)),
+                this,   SLOT(slotMovePC()) );
 
     a = actionCollection()->addAction( "run_to_cursor" );
     a->setText( i18n( "Run To Cursor" ) );
     a->setIcon( KIcon( "debug-run-cursor" ) );
-    connect(    a,      SIGNAL( triggered( bool ) ),
-                this,   SLOT( slotRunToCursor() ) );
+    connect(    a,      SIGNAL(triggered(bool)),
+                this,   SLOT(slotRunToCursor()) );
 
     a = actionCollection()->addAction( "continue" );
     a->setText( i18n( "Continue" ) );
     a->setIcon( KIcon( "media-playback-start" ) );
-    connect(    a,      SIGNAL( triggered( bool ) ),
-                m_debugView, SLOT( slotContinue() ) );
+    connect(    a,      SIGNAL(triggered(bool)),
+                m_debugView, SLOT(slotContinue()) );
 
     a = actionCollection()->addAction( "print_value" );
     a->setText( i18n( "Print Value" ) );
     a->setIcon( KIcon( "document-preview" ) );
-    connect(    a,      SIGNAL( triggered( bool ) ),
-                this, SLOT( slotValue() ) );
+    connect(    a,      SIGNAL(triggered(bool)),
+                this, SLOT(slotValue()) );
 
     // popup context m_menu
     m_menu = new KActionMenu( i18n("Debug"), this );
     actionCollection()->addAction( "popup_gdb", m_menu );
-    connect( m_menu->menu(), SIGNAL( aboutToShow() ), this, SLOT( aboutToShowMenu() ) );
+    connect( m_menu->menu(), SIGNAL(aboutToShow()), this, SLOT(aboutToShowMenu()) );
 
     m_breakpoint = m_menu->menu()->addAction(i18n("popup_breakpoint"),
-                                         this, SLOT( slotToggleBreakpoint() ) );
+                                         this, SLOT(slotToggleBreakpoint()) );
 
     QAction* popupAction = m_menu->menu()->addAction(i18n( "popup_run_to_cursor" ),
-                                                   this, SLOT( slotRunToCursor() ) );
+                                                   this, SLOT(slotRunToCursor()) );
     popupAction->setText( i18n( "Run To Cursor" ) );
     popupAction = m_menu->menu()->addAction( "move_pc",
-                                          this, SLOT( slotMovePC() ) );
+                                          this, SLOT(slotMovePC()) );
     popupAction->setText( i18nc( "Move Program Counter (next execution)", "Move PC" ) );
 
     enableDebugActions( false );
@@ -291,6 +305,7 @@ KatePluginGDBView::~KatePluginGDBView()
     mainWindow()->guiFactory()->removeClient( this );
     delete m_toolView;
     delete m_localsToolView;
+    delete m_stackToolView;
 }
 
 void KatePluginGDBView::readSessionConfig(  KConfigBase*    config,
@@ -308,13 +323,26 @@ void KatePluginGDBView::writeSessionConfig( KConfigBase*    config,
 void KatePluginGDBView::slotDebug()
 {
     QString args = m_configView->currentArgs();
+    disconnect(m_ioView, SIGNAL(stdOutText(QString)));
+    disconnect(m_ioView, SIGNAL(stdErrText(QString)));
     if ( m_configView->showIOTab() )
     {
-        args += QString( " < %1 1> %2 2> %3" )
-        .arg( m_ioView->stdinFifo() )
-        .arg( m_ioView->stdoutFifo() )
-        .arg( m_ioView->stderrFifo() );
+        connect(m_ioView, SIGNAL(stdOutText(QString)),
+                m_ioView, SLOT(addStdOutText(QString)));
+        connect(m_ioView, SIGNAL(stdErrText(QString)),
+                m_ioView, SLOT(addStdErrText(QString)));
     }
+    else {
+        connect(m_ioView, SIGNAL(stdOutText(QString)),
+                this,     SLOT(addOutputText(QString)) );
+        connect(m_ioView, SIGNAL(stdErrText(QString)),
+                this,     SLOT(addErrorText(QString)) );
+    }
+    args += QString( " < %1 1> %2 2> %3" )
+    .arg( m_ioView->stdinFifo() )
+    .arg( m_ioView->stdoutFifo() )
+    .arg( m_ioView->stderrFifo() );
+
     enableDebugActions( true );
     m_debugView->runDebugger( m_configView->currentWorkingDirectory(),
                             m_configView->currentExecutable(),
@@ -458,23 +486,22 @@ void KatePluginGDBView::enableDebugActions( bool enable )
     actionCollection()->action( "rerun"             )->setEnabled( m_debugView->debuggerRunning() );
 
     m_inputArea->setEnabled( enable );
+    m_threadCombo->setEnabled( enable );
     m_stackTree->setEnabled( enable );
     m_localsView->setEnabled( enable );
-    if ( enable )
-    {
+
+    if ( enable )  {
         m_inputArea->setFocusPolicy( Qt::WheelFocus );
-        if ( m_focusOnInput || m_configView->takeFocusAlways() )
-        {
+
+        if ( m_focusOnInput || m_configView->takeFocusAlways() ) {
             m_inputArea->setFocus();
             m_focusOnInput = false;
         }
-        else
-        {
+        else {
             mainWindow()->activeView()->setFocus();
         }
     }
-    else
-    {
+    else {
         m_inputArea->setFocusPolicy( Qt::NoFocus );
         if ( mainWindow()->activeView() ) mainWindow()->activeView()->setFocus();
     }
@@ -486,16 +513,13 @@ void KatePluginGDBView::enableDebugActions( bool enable )
         KTextEditor::MarkInterface* iface =
         qobject_cast<KTextEditor::MarkInterface*>( m_kateApplication->documentManager()->findUrl( m_lastExecUrl ) );
 
-        if (iface)
-        {
-            if ( enable )
-            {
+        if (iface) {
+            if ( enable ) {
                 iface->setMarkDescription(KTextEditor::MarkInterface::Execution, i18n("Execution point"));
                 iface->setMarkPixmap(KTextEditor::MarkInterface::Execution, KIcon("arrow-right").pixmap(10,10));
                 iface->addMark(m_lastExecLine, KTextEditor::MarkInterface::Execution);
             }
-            else
-            {
+            else {
                 iface->removeMark( m_lastExecLine, KTextEditor::MarkInterface::Execution );
             }
         }
@@ -508,6 +532,7 @@ void KatePluginGDBView::programEnded()
     m_lastExecLine = -1;
     m_stackTree->clear();
     m_localsView->clear();
+    m_threadCombo->clear();
 
     // Indicate the state change by showing the debug outputArea
     mainWindow()->showToolView( m_toolView );
@@ -563,7 +588,7 @@ void KatePluginGDBView::insertStackFrame( QString const& level, QString const& i
         m_stackTree->resizeColumnToContents(2);
         return;
     }
-
+    
     if ( level == "0")
     {
         m_stackTree->clear();
@@ -574,7 +599,7 @@ void KatePluginGDBView::insertStackFrame( QString const& level, QString const& i
     int lastSpace = info.lastIndexOf(" ");
     QString shortInfo = info.mid(lastSpace);
     columns << shortInfo;
-
+    
     QTreeWidgetItem *item = new QTreeWidgetItem(columns);
     item->setToolTip(2, QString("<qt>%1<qt>").arg(info));
     m_stackTree->insertTopLevelItem(level.toInt(), item);
@@ -589,12 +614,38 @@ void KatePluginGDBView::stackFrameChanged( int level )
 {
     QTreeWidgetItem *current = m_stackTree->topLevelItem(m_lastExecFrame);
     QTreeWidgetItem *next = m_stackTree->topLevelItem(level);
-
+    
     if ( current ) current->setIcon ( 0, QIcon() );
     if ( next )    next->setIcon( 0, KIcon("arrow-right") );
     m_lastExecFrame = level;
 }
 
+
+void KatePluginGDBView::insertThread( int number, bool active )
+{
+    if ( number < 0 ) {
+        m_threadCombo->clear();
+        m_activeThread = -1;
+        return;
+    }
+    if (!active) {
+        m_threadCombo->addItem(KIcon("").pixmap(10,10),
+                               i18n("Thread %1").arg(number), number);
+    }
+    else {
+        m_threadCombo->addItem(KIcon("arrow-right").pixmap(10,10),
+                               QString("Thread %1").arg(number), number);
+        m_activeThread = m_threadCombo->count()-1;
+    }
+    m_threadCombo->setCurrentIndex(m_activeThread);
+}
+
+void KatePluginGDBView::threadSelected( int thread )
+{
+    m_debugView->issueCommand( QString( "thread %1" ).
+    arg( m_threadCombo->itemData( thread ).toInt() ) );
+}
+
 QString KatePluginGDBView::currentWord( )
 {
     KTextEditor::View *kv = mainWindow()->activeView();
@@ -614,11 +665,31 @@ QString KatePluginGDBView::currentWord( )
     QString linestr = kv->document()->line(line);
 
     int startPos = qMax(qMin(col, linestr.length()-1), 0);
+    int lindex = linestr.length()-1;
     int endPos = startPos;
-    while (startPos >= 0 && (linestr[startPos].isLetterOrNumber() || linestr[startPos] == '_' || linestr[startPos] == '~')) {
+    while (startPos >= 0 &&
+        (linestr[startPos].isLetterOrNumber() ||
+        linestr[startPos] == '_' ||
+        linestr[startPos] == '~' ||
+        ((startPos > 1) && (linestr[startPos] == '.') && !linestr[startPos-1].isSpace()) ||
+        ((startPos > 2) && (linestr[startPos] == '>') && (linestr[startPos-1] == '-') && !linestr[startPos-2].isSpace()) ))
+    {
+        if (linestr[startPos] == '>') {
+            startPos--;
+        }
         startPos--;
     }
-    while (endPos < (int)linestr.length() && (linestr[endPos].isLetterOrNumber() || linestr[endPos] == '_')) {
+    while (endPos < (int)linestr.length() &&
+        (linestr[endPos].isLetterOrNumber() ||
+        linestr[endPos] == '_' ||
+        ((endPos < lindex-1) && (linestr[endPos] == '.') && !linestr[endPos+1].isSpace()) ||
+        ((endPos < lindex-2) && (linestr[endPos] == '-') && (linestr[endPos+1] == '>') && !linestr[endPos+2].isSpace()) ||
+        ((endPos > 1) && (linestr[endPos-1] == '-') && (linestr[endPos] == '>'))
+        ))
+    {
+        if (linestr[endPos] == '-') {
+            endPos++;
+        }
         endPos++;
     }
     if  (startPos == endPos) {
@@ -664,10 +735,25 @@ void KatePluginGDBView::showIO( bool show )
     }
 }
 
+void KatePluginGDBView::addOutputText( QString const& text )
+{
+    QScrollBar *scrollb = m_outputArea->verticalScrollBar();
+    if (!scrollb) return;
+    bool atEnd = (scrollb->value() == scrollb->maximum());
+
+    QTextCursor cursor = m_outputArea->textCursor();
+    if (!cursor.atEnd()) cursor.movePosition(QTextCursor::End);
+    cursor.insertText(text);
+
+    if (atEnd) {
+        scrollb->setValue(scrollb->maximum());
+    }
+}
+
 void KatePluginGDBView::addErrorText( QString const& text )
 {
     m_outputArea->setFontItalic( true );
-    m_outputArea->append( text );
+    addOutputText( text );
     m_outputArea->setFontItalic( false );
 }
 
